# -*- coding: utf-8 -*-

# Copyright 2010 - 2017 RhodeCode GmbH and the AppEnlight project authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import binascii
import sqlalchemy.types as types

import appenlight.lib.encryption as encryption


class BinaryHex(types.TypeDecorator):
    impl = types.LargeBinary

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = binascii.unhexlify(value)

        return value

    def process_result_value(self, value, dialect):
        if value is not None:
            value = binascii.hexlify(value)
        return value


class EncryptedUnicode(types.TypeDecorator):
    impl = types.Unicode

    def process_bind_param(self, value, dialect):
        if not value:
            return value
        return encryption.encrypt_fernet(value)

    def process_result_value(self, value, dialect):
        if not value:
            return value
        return encryption.decrypt_fernet(value)
